/*
  $Header: d:\\Personal\\cvs/aj/src/aspects/OutputStreamBuffering.java,v 1.4 2002/02/02 22:09:14 ikk Exp $ $Name:  $

  $Log: OutputStreamBuffering.java,v $
  Revision 1.4  2002/02/02 22:09:14  ikk
  Added output synchonization.

  Revision 1.3  2002/02/01 17:15:00  ikk
  Added before() advice.

  Revision 1.2  2002/02/01 06:08:14  ikk
  Added Flushing thread.

  Revision 1.1  2002/02/01 04:14:22  ikk
  Initial.

*/

package aspects;

import java.io.*;
import java.util.*;

public aspect OutputStreamBuffering implements Runnable
{
    public static final String cvs="$Id: OutputStreamBuffering.java,v 1.4 2002/02/02 22:09:14 ikk Exp $";

    private static Thread flushingThread = null;

    private static final int BUFF_SIZE=512;
    public class Buffer
    {
        byte[] buff = null;
        int counter = 0;

        Buffer()
        {
            buff = new byte[BUFF_SIZE];
        }
    }

    private Map buffTable = new HashMap();

    pointcut writeBytes(byte[] bytes):
    call(void FileOutputStream.write(byte[]))
    && args(bytes)
    && !within(OutputStreamBuffering);

    void around(byte[] bytes) throws IOException: writeBytes(bytes)
    {
        FileOutputStream out = (FileOutputStream)thisJoinPoint.getTarget();
        Buffer aBuff = (Buffer) buffTable.get(out);
        if( null == aBuff )
        {
            aBuff = new Buffer();
            buffTable.put(out, aBuff);
        }

        if(aBuff.counter + bytes.length > BUFF_SIZE)
        {
            synchronized(out)
            {
                out.write(aBuff.buff, 0, aBuff.counter);
                out.write(bytes);
                out.write(("*** buffer - "+aBuff.counter + "\n").getBytes());
            }
            
            buffTable.remove(out);
        }
        else
        {
            System.arraycopy(bytes, 0, aBuff.buff, aBuff.counter, bytes.length);
            aBuff.counter += bytes.length;
        }

        if( null == flushingThread )
        {
            flushingThread = new Thread(this);
            flushingThread.setDaemon(true);
            flushingThread.start();
        }
 
    }

    before() throws IOException : call(void FileOutputStream.close())
    {
        FileOutputStream out = (FileOutputStream)thisJoinPoint.getTarget();
        Buffer aBuff = (Buffer) buffTable.get(out);
        if( null != aBuff )
        {
            synchronized(out)
            {
                out.write(aBuff.buff, 0, aBuff.counter);
            }
            buffTable.remove(out);
        }            
    }

    public void run()
    {
        while(true)
        {
            try
            {
                Thread.sleep(3000);
                flush();
            }
            catch(Throwable e)
            {
            }
        }
    }

    void flush() throws IOException
    {
        for(Iterator i=buffTable.keySet().iterator(); i.hasNext(); )
        {
            FileOutputStream out = (FileOutputStream)i.next();
            Buffer aBuff = (Buffer) buffTable.get(out);
            synchronized(out)
            {
                out.write(aBuff.buff, 0, aBuff.counter);
                out.write(("*** flushed - "+aBuff.counter + "\n").getBytes());
            }
            i.remove();
        }
    }

    protected void finalize() throws Throwable
    {
        flush();
        super.finalize();
    }
}

