/*
  $Header: d:\\Personal\\cvs/aj/src/security/Authentication.java,v 1.7 2002/01/17 03:52:54 ikk Exp $ $Name:  $

  $Log: Authentication.java,v $
  Revision 1.7  2002/01/17 03:52:54  ikk
  Added target conditions to the startTag() pointcut.

  Revision 1.6  2002/01/16 15:14:18  ikk
  Added sessionParam pointcut and advice for it.

  Revision 1.5  2002/01/12 04:26:13  ikk
  Added new inheritance structure for UserTag and StoriesTag classes.

  Revision 1.4  2002/01/08 03:55:40  ikk
  Replaced flat file with database authentication.

  Revision 1.3  2002/01/04 18:20:42  ikk
  Replaced exception with redirection.

  Revision 1.2  2002/01/04 02:38:17  ikk
  First working version.

  Revision 1.1  2002/01/03 02:15:07  ikk
  Initial.

*/

package security;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import tags.*;

public aspect Authentication
{
    public static final String cvs="$Id: Authentication.java,v 1.7 2002/01/17 03:52:54 ikk Exp $";

    declare parents: UserTag extends TagSupport;
    declare parents: StoriesTag extends BodyTagSupport;

    pointcut startTag(): call(int *.doStartTag()) && (target(TagSupport) || target(BodyTagSupport));
    pointcut sessionParam(HttpSession sess): call(* *.*(..,HttpSession, ..)) && args(sess);

    int around() throws JspException: startTag()
    {
        PageContext pc;
        Object o = thisJoinPoint.getTarget();
        if(o instanceof TagSupport)
        {
            pc = ((TagSupport)o).getPageContext();
        }
        else if(o instanceof BodyTagSupport)
        {
            pc = ((BodyTagSupport)o).getPageContext();
        }
        else
        {
            throw new JspException("Unknown target class: "+o.getClass().getName());
        }

        String user = (String)pc.getSession().getAttribute("user");

        if( null == user )
        {
            ServletRequest req = pc.getRequest();
            user = req.getParameter("user");
            String pass = req.getParameter("pass");
            if( (null == user) || (null == pass) )
            {
                throw new NotAuthenticated();
            }

            try
            {
                if( !UsersDb.check(user, pass) )
                {
                    pc.getServletConfig().getServletContext().getRequestDispatcher("/login.jsp").forward(req, pc.getResponse());
                    return javax.servlet.jsp.tagext.Tag.SKIP_BODY;
                }
            }
            catch(Exception e)
            {
                throw new JspException(e);
            }


            pc.getSession().setAttribute("user", user);
        }

        return proceed();
    }

    before(HttpSession sess) throws JspException:  sessionParam(sess)
    {
        if( null == sess.getAttribute("user") )
        {
            throw new NotAuthenticated();
        }
    }
}
